import React, { cloneElement } from 'react';
// import '../../styles/tabs.scss'

const TYPE = {
  ADD_TAB: 'add-tab',
};
const nodeType = element => (
  element ? element.type.tabRole : undefined
);

class TabbedContent extends React.Component {
  constructor(props) {
    super(props);
    this.isStateful = !props.stateless; // used for delegating tab selections higher up (redux)

    if (this.isStateful) {
      this.state = {
        selectedTab: props.selectedTab,
      };
    }

    this.selectTab = this.selectTab.bind(this);
    this.getTabClickHandler = this.getTabClickHandler.bind(this);
  }

  render() {
    const { className } = this.props;
    return (
      <div className={`tabbed-content ${className}`}>
        {this.getChildren()}
      </div>
    );
  }

  selectTab(tabId) {
    if (this.isStateful) {
      this.setState({
        selectedTab: tabId,
      });
    }
  }

  getSelectedTab() {
    const {
      isStateful,
      state,
      props,
    } = this;
    return isStateful ? state.selectedTab : props.selectedTab;
  }

  // using id and not tab.props.tabId as this might not be set
  // in which case we would rely on auto-incremented index instead
  getTabClickHandler(tab, id) {
    if (tab.props && tab.props.type === TYPE.ADD_TAB) {
      return tab.props.tabClickHandler;
    }

    return (event) => {
      if (typeof tab.props.tabClickHandler === 'function') {
        tab.props.tabClickHandler(event);
      }

      if (!tab.props.preventDefaultTabAction) {
        this.selectTab(id);
      }
    };
  }

  getChildren() {
    const { children } = this.props;
    let tabIndex = 0;
    let tabContentIndex = 0;
    let defaultSelected;

    return React.Children.map(children, (child) => {
      if (nodeType(child) === 'TabList') {
        return cloneElement(child, {
          children: React.Children.map(child.props.children, (innerChild) => {
            if (nodeType(innerChild) === 'Tab') {
              const id = innerChild.props.tabId || tabIndex++;

              if (
                defaultSelected === undefined &&
                !this.getSelectedTab() &&
                !innerChild.props.disabled
              ) {
                defaultSelected = id;
              }

              return cloneElement(innerChild, {
                ...innerChild.props,
                id,
                selected: id === this.getSelectedTab() || defaultSelected === id,
                tabClickHandler: this.getTabClickHandler(innerChild, id),
                classList: innerChild.props.className ? innerChild.props.className.split(' ') : [],
              });
            }
            return innerChild;
          }),
        });
      }
      if (nodeType(child) === 'TabContent') {
        const id = child.props.tabId || tabContentIndex++;
        return cloneElement(child, {
          ...child.props,
          id,
          selected: id === this.getSelectedTab() || defaultSelected === id,
          classList: child.props.className ? child.props.className.split(' ') : [],
        });
      }
      return child;
    });
  }
}

TabbedContent.TYPE = TYPE;

export default TabbedContent;
